package net.sf.sido.constraint.support;

import net.sf.sido.constraint.ConstraintDefinition;
import net.sf.sido.constraint.EnumConstraint;

public class EnumConstraintParser<E extends Enum<E>> implements ConstraintParser<String> {

    protected ClassLoader getClassLoader() {
        ClassLoader cl = Thread.currentThread().getContextClassLoader();
        return cl != null ? cl : getClass().getClassLoader();
    }

	@Override
	public EnumConstraint<E> parse(ConstraintDefinition constraintDefinition) {
		String enumClassName = constraintDefinition.getParameter("enumClass", true, true, null);
		try {
			@SuppressWarnings("unchecked")
			Class<E> enumClass = (Class<E>) Class.forName(enumClassName, true, getClassLoader());
			return new EnumConstraint<E>(enumClass);
		} catch (ClassNotFoundException ex) {
			throw new IllegalArgumentException("Cannot get enum class from " + enumClassName, ex);
		}
	}

}
