package net.sf.sido.io.json;

import static net.sf.sido.factory.json.JSONUtils.getInteger;
import static net.sf.sido.factory.json.JSONUtils.getString;

import java.io.IOException;
import java.io.InputStream;

import net.sf.sido.DOJsonFieldNotArrayException;
import net.sf.sido.DataFactory;
import net.sf.sido.DataObject;
import net.sf.sido.DataType;
import net.sf.sido.Property;
import net.sf.sido.io.AbstractDataObjectReader;
import net.sf.sido.support.DOCache;

import org.codehaus.jackson.JsonNode;
import org.codehaus.jackson.JsonParser;
import org.codehaus.jackson.map.ObjectMapper;

public class JSONDataObjectReader extends AbstractDataObjectReader<JsonNode> {

	@Override
	protected int getDOId(JsonNode e) {
		return getInteger(e, JSONDOSchema.ID, false, 0);
	}

	@Override
	protected int getDOReference(JsonNode e) {
		return getInteger(e, JSONDOSchema.REF, false, 0);
	}

	@Override
	protected String getDOType(JsonNode e, boolean required) {
		return getString(e, JSONDOSchema.TYPE, required, null);
	}

	@Override
	protected JsonNode getElement(JsonNode e, Property<?> property) {
		return e.get(property.getName());
	}

	@Override
	protected Iterable<JsonNode> getElements(JsonNode e, Property<?> property) {
		String name = property.getName();
		JsonNode jArray = e.get(name);
		if (jArray == null || jArray.isNull()) {
			return null;
		} else if (!jArray.isArray()) {
			throw new DOJsonFieldNotArrayException(name);
		} else {
			return jArray;
		}
	}

	@Override
	protected String getText(JsonNode eNode) {
		return eNode.getValueAsText();
	}

	@Override
	public DataObject read(InputStream in, DataFactory factory, DataType expectedType) throws IOException {
		// Reads the Json
		ObjectMapper mapper = new ObjectMapper();
		mapper.configure(JsonParser.Feature.ALLOW_UNQUOTED_FIELD_NAMES, true);
		mapper.configure(JsonParser.Feature.ALLOW_SINGLE_QUOTES, true);
		mapper.configure(JsonParser.Feature.ALLOW_COMMENTS, true);
		JsonNode jRoot = mapper.readValue(in, JsonNode.class);
		// Reads from the JSON model
		return read(new DOCache(), jRoot, factory, expectedType);
	}

	@Override
	protected Object simple(JsonNode jNode, Class<?> type, DataFactory factory) {
		// Number
		if (Number.class.isAssignableFrom(type)) {
			return jNode.getNumberValue();
		}
		// Boolean class
		else if (Boolean.class.isAssignableFrom(type)) {
			return jNode.getBooleanValue();
		}
		// String
		else if (String.class.isAssignableFrom(type)) {
			return jNode.getTextValue();
		}
		// Any other type
		else {
			return super.simple(jNode, type, factory);
		}
	}
}
