package net.sf.sido.io.json;

import java.io.IOException;
import java.io.OutputStream;
import java.lang.reflect.Method;
import java.util.List;

import net.sf.sido.DataObject;
import net.sf.sido.DataType;
import net.sf.sido.Property;
import net.sf.sido.io.AbstractDataObjectWriter;
import net.sf.sido.io.DOReferenceManagement;

import org.codehaus.jackson.JsonNode;
import org.codehaus.jackson.node.ArrayNode;
import org.codehaus.jackson.node.JsonNodeFactory;
import org.codehaus.jackson.node.NumericNode;
import org.codehaus.jackson.node.ObjectNode;

import com.google.common.primitives.Primitives;

public class JSONDataObjectWriter extends AbstractDataObjectWriter<JsonNode, String> {

	@Override
	protected void add(JsonNode e, String propertyName, JsonNode propertyValue) {
		((ObjectNode) e).put(propertyName, propertyValue);
	}

	@Override
	protected void createArray(JsonNode e, String pName, List<JsonNode> elements) {
		ArrayNode array = ((ObjectNode) e).putArray(pName);
		for (JsonNode objectNode : elements) {
			array.add(objectNode);
		}
	}

	@Override
	protected ObjectNode createElement(String name) {
		return JsonNodeFactory.instance.objectNode();
	}

	@Override
	protected JsonNode createValue(DataObject o, String pName, Object value) {
		// Value type
		Class<?> type = value.getClass();
		// Number class?
		if (value instanceof Number) {
			try {
				Class<?> primitiveType = Primitives.unwrap(type);
				Method numberNodeMethod = JsonNodeFactory.class.getMethod("numberNode", primitiveType);
				NumericNode node = (NumericNode) numberNodeMethod.invoke(JsonNodeFactory.instance, value);
				return node;
			} catch (Exception ex) {
				throw new DOJsonCannotFormatException(ex, value, type.getName());
			}
		}
		// Boolean class
		else if (value instanceof Boolean) {
			return JsonNodeFactory.instance.booleanNode((Boolean) value);
		}
		// String
		else if (value instanceof String) {
			return JsonNodeFactory.instance.textNode((String) value);
		}
		// Any other type
		else {
			String string = getText(o.getFactory(), value);
			return JsonNodeFactory.instance.textNode(string);
		}
	}

	@Override
	protected String getDOArrayName() {
		return "unused";
	}

	@Override
	protected String getPropertyName(JsonNode e, Property<?> property) {
		return property.getName();
	}

	@Override
	protected String getSimpleArrayName() {
		return "unused";
	}

	@Override
	protected void setDOId(JsonNode e, int id) {
		((ObjectNode) e).put(JSONDOSchema.ID, id);
	}

	@Override
	protected void setDOReference(JsonNode e, int id) {
		((ObjectNode) e).put(JSONDOSchema.REF, id);
	}

	@Override
	protected void setDOType(JsonNode e, DataType type) {
		((ObjectNode) e).put(JSONDOSchema.TYPE, type.getQualifiedName());
	}

	@Override
	public void write(OutputStream out, DataObject o, DOReferenceManagement referenceManagement) throws IOException {
		// Creates the JSON node
		JsonNode jRoot = createElement(null, o, referenceManagement);
		// Writes the JSON in the output
		out.write(jRoot.toString().getBytes("UTF-8"));
	}

}
